import sys, os
import urlparse, urllib, json
from lib import IPhoenix
from lib.WatchedManager import WatchedManager
from lib.FavoriteManager import FavoriteManager
from lib.PhoenixInstaller import PhoenixInstaller
from plugins import Plugins
from plugins.video import IVideo
from resolver.Resolver import Resolver
from extended.ui import InputWindow
from extended.player import XBMCPlayer
import zlib
try:
    import xbmc, xbmcplugin, xbmcgui, xbmcaddon
except:
    from dev.xbmclib import *

addon = xbmcaddon.Addon('plugin.video.phoenix')
profile = xbmc.translatePath(addon.getAddonInfo('profile'))
home = addon.getAddonInfo('path')
plugins = Plugins(xbmc.translatePath(os.path.join(home,"plugins")))

def decodeParams(params=None):
    if params == None:
        params = str(sys.argv[2]).replace("?","")
    elif not params.startswith("j="):
        params = params.replace("%s?"%sys.argv[0],"")
        if not params.startswith("j="):
            params = "j="+params
    prs = urlparse.parse_qs(params, True)
    try:
        return json.loads(zlib.decompress(prs.get("j",["e30="])[0].decode('base64','strict')))
    except:
        return {}

def encodeParams(obj):
    try:
        return "%s?%s"%(sys.argv[0],urllib.urlencode({"j":zlib.compress(json.dumps(obj,ensure_ascii=True,sort_keys=True)).encode('base64','strict')}));
    except:
        return sys.argv[0]
    
def getJsonString(pluginUrl):
    try:
        return pluginUrl.replace(sys.argv[0]+"?j=","")
    except:
        return ""

params = decodeParams()
print params
print str(sys.argv[2])
path = params.get("path","")
args = params.get("args",{})
play = params.get("play",None)
watchedmgr = WatchedManager()
favmgr = FavoriteManager()

def processDir(settings,totalItems=None):
    global watchedmgr,path
    contextMenuItems = []
    ok = True
    u = encodeParams(settings)
    infoLabels = settings.get("infoLabels",None)
    if infoLabels == None: infoLabels = { "title": settings.get("title","No title") }
    iconImage = infoLabels.get("cover_url","")
    if iconImage == "":
        iconImage = infoLabels.get("poster","")
        if iconImage == "":
            iconImage = settings.get("iconImage","DefaultFolder.png")
    
    thumbnailImage = infoLabels.get("banner_url","")
    if thumbnailImage == "":
        thumbnailImage = infoLabels.get("cover_url","")
        if thumbnailImage == "":
            thumbnailImage = infoLabels.get("poster","")
            if thumbnailImage == "":
                thumbnailImage = settings.get("thumbnailImage","DefaultFolder.png")
    
    liz = xbmcgui.ListItem(settings.get("title",infoLabels.get("title","No title")), iconImage=iconImage, thumbnailImage=thumbnailImage)
    if infoLabels.get("backdrop_url","")!="":
        liz.setProperty('fanart_image', infoLabels.get("backdrop_url"))
    
    infoLabels['overlay'] = 6
    
    if settings.get("play",None) != None:
        settings["isFolder"] = False
        settings["watched"] = False
        settings["favmgrourites"] = False

    if settings.get("mid","") != "" or settings.get("sid","") != "":
        mid = settings.get("mid","")
        sid = settings.get("sid","")
        playcount = watchedmgr.getPlayCount(mid,sid)
        infoLabels['playcount'] = playcount
        if playcount > 0:   infoLabels['overlay'] = 7
            
        if settings.get("showWatched",True) == True:
            if playcount == 0:
                contextMenuItems.append(('Mark as Watched', 'XBMC.RunPlugin(%s)' % encodeParams({"path":"/markaswatched","mid":mid,"sid":sid})))
            else:
                contextMenuItems.append(('Mark as Unwatched', 'XBMC.RunPlugin(%s)' % encodeParams({"path":"/markasunwatched","mid":mid,"sid":sid})))
        
        if settings.get("showFavourite",True) == True:
            dat = getJsonString(u)
            # generate id from path,mid,sid
            favid = favmgr.generateId(settings.get("path","-"),settings.get("mid","-"),settings.get("sid","-"))
            if favmgr.isFavourite(favid=favid,jsonString=dat):
                contextMenuItems.append(('Remove from favourites', 'XBMC.RunPlugin(%s)' % encodeParams({"path":"/removefromfavourites","favid":favid,"jsonString":dat})))
            else:
                contextMenuItems.append(('Add to favourites', 'XBMC.RunPlugin(%s)' % encodeParams({"path":"/addtofavourites","favid":favid,"bookmark_path":settings.get("path","-"),"jsonString":dat})))

    if len(infoLabels) > 3:
        contextMenuItems.append(('Show Information', 'XBMC.Action(Info)'))
    # its a TV-Show Season
    if settings.get("sid","") != "" and settings.get("sid","") == "":
        liz.setProperty('UnWatchedEpisodes', str(watchedmgr.getUnWatchedEpisodes(settings.get("sid",""))))
        liz.setProperty('WatchedEpisodes', str(watchedmgr.getWatchedEpisodes(settings.get("sid",""))))
        
    if contextMenuItems:
        liz.addContextMenuItems(contextMenuItems, replaceItems=True)
    
    liz.setInfo("video", infoLabels)
    
    ok = xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]), url=u, listitem=liz, isFolder=settings.get("isFolder",True), totalItems=totalItems)
    return ok

#IPhoenix.logNotice("url: %s" % sys.argv[2])
IPhoenix.logNotice("Path: %s" % path)
IPhoenix.logNotice("Args: %s" % args)
IPhoenix.logNotice("Play: %s" % play)

if play != None:
    url = None
    domain = None
    mid = None
    
    domain = None
    if type(play) is dict:
        url = play.get("url",None)
        domain = play.get("domain",None)
        mid = play.get("mid",None)
    else:
        url = play
        
    dialog = xbmcgui.DialogProgress()
    dialog.create('Resolving', 'Resolving Link...')
    dialog.update(0)
    if not dialog.iscanceled():
        try:
            res = Resolver(url=url,domain=domain,mid=mid)
            dialog.update(50)
            if not dialog.iscanceled():
                streamurl = res.getStream()
                hoster = res.getHoster()
                if streamurl != None and not dialog.iscanceled():
                    dialog.update(100)
                    dialog.close()
                    if not dialog.iscanceled():
                        player = XBMCPlayer(id=hoster.getURL())
                        def playedCallback():
                            if player.isWatched() == True:
                                watchedmgr.markAsWatched(params.get("mid",""), params.get("sid",""))
                                xbmc.executebuiltin("XBMC.Container.Refresh")                 
                        player.onPlayBackStopped(playedCallback)
                        player.onPlayBackEnded(playedCallback)
                        player.play(streamurl)
        except Exception as e:
            okdialog = xbmcgui.Dialog()
            IPhoenix.logError(e)
            ok = okdialog.ok("Error", "%s" % e)
    dialog.close()
elif path.endswith("/search") and len(path) > 7: #search in plugin
    plugin = plugins.getPlugin(path)
    if plugin != None and plugin.isSearchable == True:
        kb = xbmc.Keyboard('', 'Search '+plugin.getTitle().capitalize(), False)
        kb.doModal()
        if (kb.isConfirmed()):
            search = kb.getText()
            if search != '':
                directories = plugins.searchPlugin(path,search)
                totalItems=len(directories)
                for directory in directories:
                    processDir(directory,totalItems)
elif path == "/markaswatched":
    watchedmgr.markAsWatched(params.get("mid",""), params.get("sid",""))
    xbmc.executebuiltin("XBMC.Container.Refresh")
elif path == "/markasunwatched":
    watchedmgr.markAsUnWatched(params.get("mid",""), params.get("sid",""))
    xbmc.executebuiltin("XBMC.Container.Refresh")
elif path == "/addtofavourites":
    favmgr.append(favid=params.get("favid"), path=params.get("bookmark_path"), jsonString=params.get("jsonString"))
    xbmc.executebuiltin("XBMC.Container.Refresh")
elif path == "/removefromfavourites":
    favmgr.remove(favid=params.get("favid"), jsonString=params.get("jsonString"))
    xbmc.executebuiltin("XBMC.Container.Refresh")
elif path.endswith("/openfavourites") and len(path) > 15: # display plugin favmgrorites
    plugindirectory = path.split("/")
    path = plugindirectory.pop(0)
    name = plugindirectory.pop(0)
    favs = favmgr.getFavourites(path=path+"/"+name)
    totalItems=len(favs)
    for fav in favs:
        settings = decodeParams(fav)
        processDir(settings,totalItems)
else:
    if path == "":
        # check if data needs to be install
        pinst = PhoenixInstaller(os.path.join(home,"./install/install.bin"))
        if pinst.isInstallNeeded():
            dialog = xbmcgui.DialogProgress()
            dialog.create('Installing Package', 'installing...')
            dialog.update(0)
            for status in pinst.install():
                dialog.update(status[1],status[0])
            dialog.close()
    
    directories = plugins.getDirectory(path,args)
    if path == "" and len(directories) == 1: # if only one main category is present open main category directly
        directories = plugins.getDirectory(directories[0].get("path",""),directories[0].get("args",{}))
        
    if len(path.split("/")) == 2: # is plugin root
        plugindirectory = path.split("/")
        path = plugindirectory.pop(0)
        name = plugindirectory.pop(0)
        plugin = plugins.getPlugin()
        if plugin and plugin.isSearchable:
            directories.append(IPhoenix.dictMerge(plugins.getPluginDefaults(plugin),{"path":"%s/%s/search"%(path,name),"title":"Search", "isFolder":True}));
        if plugin:
            directories.append(IPhoenix.dictMerge(plugins.getPluginDefaults(plugin),{"path":"%s/%s/openfavourites"%(path,name),"title":"Favourites", "isFolder":True}));
        
    
    totalItems=len(directories)
    for directory in directories:
        processDir(directory,totalItems)
    
xbmcplugin.endOfDirectory(int(sys.argv[1]))