import os, hashlib, StringIO
from lib import IPhoenix

class FavoriteManager(IPhoenix):
    dbname = os.path.join("core","FavoriteManager.db")
    
    def __init__(self):
        IPhoenix.__init__(self)
        
    def initDatabase(self, cursor):
        IPhoenix.initDatabase(self, cursor)
        cursor.execute('CREATE TABLE IF NOT EXISTS favourites (favid VARCHAR(40) PRIMARY KEY,path VARCHAR(1024),data);')
        cursor.execute('CREATE INDEX IF NOT EXISTS idx_path ON favourites (path);')
        
    def isFavourite(self,favid=None,jsonString=""):
        if favid==None:
            favid = self.generateId(jsonString)
        else:
            favid = self.generateId(favid)
        cur = self.getDBCursor()
        cur.execute("SELECT count(*) FROM favourites WHERE favid=?",(favid,))
        row = cur.fetchone()
        if row and int(row[0]) > 0:
            return True
        return False
    
    def append(self,favid=None,path=None,jsonString=""):
        if favid==None:
            favid = self.generateId(jsonString)
        else:
            favid = self.generateId(favid)
        cur = self.getDBCursor()
        cur.execute("INSERT OR REPLACE INTO favourites (favid,path,data) VALUES(?,?,?)",(favid,path,jsonString))
        
    def remove(self,favid=None,jsonString=""):
        if favid==None:
            favid = self.generateId(jsonString)
        else:
            favid = self.generateId(favid)
        cur = self.getDBCursor()
        cur.execute("DELETE FROM favourites WHERE favid=?",(favid,))
    
    def getFavourites(self,path=""):
        retVal = []
        cur = self.getDBCursor()
        if path == "":
            cur.execute("SELECT data FROM favourites")
        else:
            cur.execute("SELECT data FROM favourites WHERE path LIKE ?",(path+"%",))
        for row in cur:
            retVal.append(row[0])
        return retVal
    
    def generateId(self, *args):
        hashThis = StringIO.StringIO()
        for arg in args:
            hashThis.write(str(arg))
        hashString = hashThis.getvalue()
        hashThis.close()
        return hashlib.sha1(hashString).hexdigest()        