import re
import string

def _fullMatch():
    return 


def extract(source):
    """Unpacks all P.A.C.K.E.R. packed js strings from source-string."""
    list = []
    repacker = "(eval\(function\(\s*[a-zA-Z0-9]+\s*,\s*[a-zA-Z0-9]+\s*,\s*[a-zA-Z0-9]+\s*,\s*[a-zA-Z0-9]+\s*,\s*[a-zA-Z0-9]+\s*,\s*[a-zA-Z0-9]+\s*\)\{.*?\}\(\s*'.*?;'\s*,\s*[0-9]+\s*,\s*[0-9]+\s*,\s*'.*?'\.split\('\|'\).*\).*\))"
    all = re.findall(repacker, source, re.I + re.M)
    cnt = -1
    for match in all:
        try:            
            unpacked = unpack(match)
            if unpacked != None:
                cnt = cnt + 1
                list.insert(cnt,unpacked)
        except:
            pass
    return list

def unpack(source):
    """Unpacks P.A.C.K.E.R. packed js code."""
    payload, symtab, radix, count = _filterargs(source)

    if count != len(symtab):
        raise UnpackingError('Malformed p.a.c.k.e.r. symtab.')

    try:
        unbase = Unbaser(radix)
    except TypeError:
        raise UnpackingError('Unknown p.a.c.k.e.r. encoding.')

    def lookup(match): #Look up symbols in the synthetic symtab.
        word  = match.group(0)
        return symtab[unbase(word)] or word

    source = re.sub(r'\b\w+\b', lookup, payload)
    return _replacestrings(source)

def _filterargs(source):
    """Juice from a source file the four args needed by decoder."""
    fullparam = "}\('(.*)', *(\d+), *(\d+), *'(.*)'\.split\('\|'\), *(\d+), *(.*)\) *\)"
    missparam = "}\('(.*)', *(\d+), *(\d+), *'(.*)'\.split\('\|'\) *\) *\)"
    if re.search(fullparam, source, re.DOTALL + re.I):
        args = re.search(fullparam, source, re.DOTALL + re.I).groups() 
    else:
        args = re.search(missparam, source, re.DOTALL + re.I).groups()

    try:
        return args[0], args[3].split('|'), int(args[1]), int(args[2])
    except ValueError:
        #raise UnpackingError('Corrupted p.a.c.k.e.r. data.')
        pass

def _replacestrings(source):
    """Strip string lookup table (list) and replace values in source."""
    match = re.search(r'var *(_\w+)\=\["(.*?)"\];', source, re.DOTALL)
    if match:
        varname, strings = match.groups()
        startpoint = len(match.group(0))
        lookup = strings.split('","')
        variable = '%s[%%d]' % varname
        for index, value in enumerate(lookup):
            source = source.replace(variable % index, '"%s"' % value)
        return source[startpoint:]
    return source

class UnpackingError(Exception):
    pass

class Unbaser(object):
    """Functor for a given base. Will efficiently convert
    strings to natural numbers."""
    ALPHABET = { 36 : '0123456789abcdefghijklmnopqrstuvwxyz',62:'0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', 95:' !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~'}

    def __init__(self, base):
        self.base = base

        # If base can be handled by int() builtin, let it do it for us
        if 2 <= base <= 36:
            self.unbase = lambda string: int(string, base)
        else:
            # Build conversion dictionary cache
            try:
                if self.ALPHABET.get(base,None) != None:
                    self.dictionary = dict((cipher, index) for 
                        index, cipher in enumerate(self.ALPHABET[base]))
                else:
                    # handle in between dicts 
                    self.dictionary = dict((cipher, index) for 
                        index, cipher in enumerate(string.printable[:base]))
            except KeyError:
                raise TypeError('Unsupported base encoding.')

            self.unbase = self._dictunbaser

    def __call__(self, string):
        return self.unbase(string)

    def _dictunbaser(self, string):
        """Decodes a  value to an integer."""
        ret = 0
        for index, cipher in enumerate(string[::-1]):
            ret += (self.base ** index) * self.dictionary[cipher]
        return ret

# Demo
#print unpack("eval(function(p,a,c,k,e,d){e=function(c){return(c<a?'':e(parseInt(c/a)))+((c=c%a)>35?String.fromCharCode(c+29):c.toString(36))};if(!''.replace(/^/,String)){while(c--){d[e(c)]=k[c]||e(c)}k=[function(e){return d[e]}];e=function(){return'\\w+'};c=1};while(c--){if(k[c]){p=p.replace(new RegExp('\\b'+e(c)+'\\b','g'),k[c])}}return p}('a(\'d\').p({\'o\':\'0://d.9.1/n.8.q\',\'r\':\'0://m.3-4.1/3/c.t\',\'s\':\'0://f.3-4.1/7/v/j/3/c.g\',\'h\':\'0://i.9.1/7/k/e/5/e.l\',\'u\':\'G\',\'L\':\'0\',\'0.K\':\'w\',\'I\':\'N\',\'P\':\'Q\',\'O\':\'M\',\'H\':{\'A-2\':{}}});z 6=b;a().y(x(){B(6==b)$.C(\'/F.E\',{D:\'0://f.3-4.1\'});6=J});',53,53,'http|com||2gb|hosting||played|files||longtailvideo|jwplayer|false|62dbe0abded72df|player|bekle|gb3|flv|skin|www|4ebbfbca|skins|zip|images|player5|flashplayer|setup|swf|image|file|jpg|stretching|46f876cb9e956f0e6a0675ed625a450d|start|function|onPlay|var|timeslidertooltipplugin|if|post|server|php|track|exactfit|plugins|controlbar|true|startparam|provider|348|bottom|height|width|620'.split('|'),0,{}))")
#print unpack("eval(function(p,a,c,k,e,d){while(c--)if(k[c])p=p.replace(new RegExp('\\b'+c.toString(a)+'\\b','g'),k[c]);return p}('1k.1j(\'<b 1i=\"1h:1g; a:4%; 9:4%\"a=\"4%\"9=\"4%\"x=\"1f\"1e=\"1d:1c-1b-1a-19-18\"17=\"6://f.7.5/e/16.15\"><2 1=\"n\"0=\"m\"/><2 1=\"l\"0=\"14\"/><2 1=\"8\"0=\"\"/><2 1=\"i\"0=\"3\"><2 1=\"j\"0=\"3\"/><2 1=\"h\"0=\"g\"/><2 1=\"k\"0=\"3\"/><2 1=\"w\"0=\"6://v.u.5:t/d/s/r-q-p.o\"/><c x=\"13\"12=\"11/7\"w=\"6://v.u.5:t/d/s/r-q-p.o\"n=\"m\"l=\"10\"a=\"4%\"9=\"4%\"8=\"\"k=\"3\"j=\"3\"8=\"\"i=\"3\"h=\"g\"z=\"6://f.7.5/e/y/\"></c></b>\');',36,57,'value|name|param|false|100|com|http|divx|previewImage|height|width|object|embed||plugin|go|Play|previewMessage|allowContextMenu|bannerEnabled|autoPlay|wmode|Stage6|custommode|avi|s10e01|stargatesg1|itg|tmgelungvsulzrqmy3n37bnbbgrxh4ojia7wjiwgk6h6qpzbk7sysd52|182|uploadc|www14|src|id|download|pluginspage|transparent|video|type|np_vid|opaque|cab|DivXBrowserPlugin|codebase|CC0F21721616|9C46|41fa|D0AB|67DABFBF|clsid|classid|ie_vid|black|background|style|write|document'.split('|')))")
#print unpack("eval(function(p,a,c,k,e,d){e=function(c){return(c<a?'':e(parseInt(c/a)))+((c=c%a)>35?String.fromCharCode(c+29):c.toString(36))};if(!''.replace(/^/,String)){while(c--){d[e(c)]=k[c]||e(c)}k=[function(e){return d[e]}];e=function(){return'\\w+'};c=1};while(c--){if(k[c]){p=p.replace(new RegExp('\\b'+e(c)+'\\b','g'),k[c])}}return p}('a(\'d\').p({\'o\':\'0://d.9.1/n.8.q\',\'r\':\'0://m.3-4.1/3/c.t\',\'s\':\'0://f.3-4.1/7/v/j/3/c.g\',\'h\':\'0://i.9.1/7/k/e/5/e.l\',\'u\':\'G\',\'L\':\'0\',\'0.K\':\'w\',\'I\':\'N\',\'P\':\'Q\',\'O\':\'M\',\'H\':{\'A-2\':{}}});z 6=b;a().y(x(){B(6==b)$.C(\'/F.E\',{D:\'0://f.3-4.1\'});6=J});',53,53,'http|com||2gb|hosting||played|files||longtailvideo|jwplayer|false|62dbe0abded72df|player|bekle|gb2|flv|skin|www|50461ba4|skins|zip|images|player5|flashplayer|setup|swf|image|file|jpg|stretching|b213822fe612cb6159bd35a1f2f36dfa|start|function|onPlay|var|timeslidertooltipplugin|if|post|server|php|track|exactfit|plugins|controlbar|true|startparam|provider|348|bottom|height|width|620'.split('|'),0,{}))")
