import os,tarfile,hashlib,shutil
try:
    import xbmc, xbmcplugin, xbmcgui, xbmcaddon
except:
    from dev.xbmclib import *
try:
    from sqlite3 import dbapi2 as sqlite
    # print "Loading sqlite3 as DB engine"
except:
    from pysqlite2 import dbapi2 as sqlite
    # print "Loading sqlite3 as DB engine"

class PhoenixInstaller(object):
    source = None
    installedHash = None
    installNeeded = False
    tar = None
    installedHashFile = "special://database/phoenix/installed.hash"
    srcFolderString = "special://database/phoenix/import/"
    
    def __init__(self,source):
        if not os.path.exists(source): raise IOError("%s does not exists!" % source)
        self.source = source
        self.ensureDir(xbmc.translatePath(self.installedHashFile))
        self.installedHash = self.__getHash(xbmc.translatePath(self.installedHashFile))
        self.toInstallHash =  self.__getHash(self.source)
        self.installNeeded = (self.installedHash != self.toInstallHash)
        if self.installNeeded:
            self.tar = tarfile.open(self.source, "r")
            self.srcFolder = xbmc.translatePath(self.srcFolderString)
            self.ensureDir(self.srcFolder)
    
    def ensureDir(self,f):
        d = os.path.dirname(f)
        if not os.path.exists(d):
            os.makedirs(d)
    
    def __getHash(self,file):
        if file.endswith(".hash") and os.path.exists(file):
            f = open(file,"r")
            hash = f.readline().replace("\n","").replace("\r","")
            f.close()
            return hash
        elif not os.path.exists(file+".hash") and os.path.exists(file):
            return self.__sha1Checksum(file)
        elif os.path.exists(file+".hash"):
            f = open(file+".hash","r")
            hash = f.readline().replace("\n","").replace("\r","")
            f.close()
            return hash
        else:
            return None
            
    def __sha1Checksum(self,file):
        fh = open(file, 'rb')
        m = hashlib.sha1()
        while True:
            data = fh.read(8192)
            if not data: break
            m.update(data)
        return m.hexdigest()
    
    
    def __mergeDB(self,dst,src):
        if not os.path.exists(src): raise IOError("%s does not exists!" % src)
        try:
            srcCon = sqlite.connect(src)
            srcCon.text_factory = str
            dstCon = sqlite.connect(dst)
            dstCon.text_factory = str
            srcCur = srcCon.cursor()
            srcCur.execute('PRAGMA temp_store = MEMORY;')
            srcCur.execute('PRAGMA synchronous = OFF;')
            dstCur = dstCon.cursor()
            dstCur.execute('PRAGMA temp_store = MEMORY;')
            dstCur.execute('PRAGMA synchronous = OFF;')
            
            def listDiff(list1, list2):
                """uses list1 as the reference, returns list of items not in list2"""
                diff_list = []
                for item in list1:
                    if not item in list2:
                        diff_list.append(item)
                return diff_list
    
            def getTableAlterADD(tableName,column):
                srcRes = srcCur.execute("PRAGMA table_info('%s')" % tableName)
                for cid, name, type, notnull, default, pk in srcRes.fetchall():
                    if name == column:
                        sql = "ALTER TABLE %s ADD COLUMN %s %s" % (tableName,column,type)
                        if default != None:
                            if str(default).isnumeric():
                                sql = sql + " DEFAULT %s" % default
                            else:
                                sql = sql + " DEFAULT '%s'" % default.replace("'","\\'")
                        if int(notnull) > 0:
                            sql = sql + " NOT NULL"
                        if int(pk) > 0:
                            sql = sql + " PRIMARY KEY"
                        return sql + ";"
                
            
            dstIndexSchemaRes = dstCur.execute("SELECT name, type, sql FROM sqlite_master WHERE sql NOT NULL AND type IN ('index', 'trigger', 'view')")
            # drop source indexes, triggers, views
            for name, type, sql in dstIndexSchemaRes.fetchall():  dstCur.execute("DROP %s IF EXISTS %s" %(type.upper(),name))
            
            # gater structure info
            srcSchemaRes = srcCur.execute("SELECT name, type, sql FROM sqlite_master WHERE sql NOT NULL AND type == 'table'")
            dstSchemaRes = dstCur.execute("SELECT name, type, sql FROM sqlite_master WHERE sql NOT NULL AND type == 'table'")
            
            srcStructure = {}
            for tableName, type, sql in srcSchemaRes.fetchall():
                srcRes = srcCur.execute("PRAGMA table_info('%s')" % tableName)
                srcColumnNames = [str(tableInfo[1]) for tableInfo in srcRes.fetchall()]
                srcStructure[tableName]={"sql":sql.replace("CREATE TABLE ","CREATE TABLE IF NOT EXISTS "),"columns":srcColumnNames}
                
            dstStructure = {}
            for tableName, type, sql in dstSchemaRes.fetchall():
                dstRes = dstCur.execute("PRAGMA table_info('%s')" % tableName)
                dstColumnNames = [str(tableInfo[1]) for tableInfo in dstRes.fetchall()]
                dstStructure[tableName]={"sql":sql.replace("CREATE TABLE ","CREATE TABLE IF NOT EXISTS "),"columns":dstColumnNames}
            
            # create dst indexes
            srcIndexSchemaRes = srcCur.execute("SELECT name, type, tbl_name, sql FROM sqlite_master WHERE sql NOT NULL AND type IN ('index', 'trigger', 'view')")
            srcIndexStructure = {}
            for name, type, tblName, sql in srcIndexSchemaRes.fetchall():
                if srcIndexStructure.get(tblName,None) == None: 
                    srcIndexStructure[tblName] = []
                srcIndexStructure[tblName].append(sql)
            
            for srcTableName in srcStructure:
                if dstStructure.get(srcTableName,None) == None: # table does not extist so create it
                    dstCur.execute(srcStructure[srcTableName]["sql"])
                else: # table exist's update structure                    
                    missingColumns = listDiff(srcStructure[srcTableName]["columns"],dstStructure.get(srcTableName,{}).get("columns",[]))
                    if len(missingColumns) > 0:
                        for column in missingColumns:
                            dstCur.execute(getTableAlterADD(srcTableName,column))
                # create indexes
                for index in srcIndexStructure.get(srcTableName,[]): dstCur.execute(index)
                # insert data
                columnNames = ",".join(srcStructure[srcTableName]["columns"])
                columnValues = ",".join([str("?") for culumn in srcStructure[srcTableName]["columns"]])
                srcRes = srcCur.execute("SELECT * FROM %s" % (srcTableName))
                for res in srcRes: dstCur.execute("INSERT OR REPLACE INTO %s (%s) VALUES(%s)" %(srcTableName,columnNames,columnValues),tuple(res))
            
            srcCon.commit()
            srcCur.close()
            dstCur.close()
            srcCon.close()
            dstCon.close()
            return True
        except:
            try:
                dstCur.close()
                dstCon.close()
            except:
                pass
            os.remove(dst)
            os.rename(src, dst)
            return True
    
    def install(self):
        if not self.installNeeded:
            yield ("noting to do.",100)
        else:
            mlen = len(self.tar.getnames())
            cnt = 0
            for member in self.tar.getnames():
                cnt=cnt+1
                dst = xbmc.translatePath("special://database/phoenix/%s"%member)
                yield ("extracting: %s" % member,int(float(cnt/(mlen/100.0))))
                self.tar.extract(member, path=self.srcFolder)
                src = os.path.normpath(os.path.join(self.srcFolder,member))
                yield ("merging: %s" % member,int(float(cnt/(mlen/100.0))))
                if self.__mergeDB(dst,src): 
                    os.remove(src)
                    yield ("imported: %s" % member,int(float(cnt/(mlen/100.0))))
            self.tar.close()
            shutil.rmtree(self.srcFolder)
            f = open(xbmc.translatePath(self.installedHashFile),"w")
            f.write(self.toInstallHash)
            f.close()
    def isInstallNeeded(self):
        return self.installNeeded
    
class PhoenixInstallerGenerator(object):
    target = None
    tar = None
    
    def __init__(self,target):
        self.target = target
        self.tar = tarfile.open(self.target, "w:gz")
        
    def __del__(self):
        self.tar.close()
        src = self.target+".hash"
        f = open(src,"w")
        f.write(self.__sha1Checksum())
        f.close()
    
    def __sha1Checksum(self):
        fh = open(self.target, 'rb')
        m = hashlib.sha1()
        while True:
            data = fh.read(8192)
            if not data: break
            m.update(data)
        return m.hexdigest()
    
    @staticmethod
    def reset(tarinfo):
        tarinfo.uid = tarinfo.gid = tarinfo.nlink = 0
        tarinfo.uname = tarinfo.gname = "root"
        return tarinfo
    
    def add(self,source,arcname=None):
        if arcname == None: 
            self.tar.add(source,filter=PhoenixInstallerGenerator.reset,arcname=os.path.basename(source))
        else:
            self.tar.add(source,filter=PhoenixInstallerGenerator.reset,arcname=arcname)
            