import os, hashlib
from lib import IPhoenix

class WatchedManager(IPhoenix):
    dbname = os.path.join("core","WatchedManager.db")
    
    def __init__(self):
        IPhoenix.__init__(self)
        
    def initDatabase(self, cursor):
        IPhoenix.initDatabase(self, cursor)
        cursor.execute('CREATE TABLE IF NOT EXISTS watched (mid VARCHAR(255),sid VARCHAR(255),playcount INTEGER DEFAULT 0, PRIMARY KEY (mid, sid));')
        
    def getPlayCount(self,mid="",sid=""):
        """ mid = media id 
            sid = series id => only for TV Series
        """
        if mid == None: mid = ""
        if sid == None: sid = ""
        mid = mid.encode("utf-8")
        sid = sid.encode("utf-8")
            
        retVal = 0
        cur = self.getDBCursor()
        cur.execute("SELECT playcount FROM watched WHERE mid=? AND sid=?",(mid,sid))
        row = cur.fetchone()
        if row != None:
            retVal = int(row[0])
        else:
            if sid != "" and mid=="":
                cur.execute("SELECT playcount FROM watched WHERE sid=? AND mid = ''",(sid,))
                res = cur.fetchone()
                if res and int(cur.fetchone()[0]) > 0: 
                    # if season is marked as played unmark because of new episode
                    cur.execute("UPDATE watched SET playcount=0 WHERE mid='' AND sid=?",(sid,))
                else:
                    cur.execute("INSERT OR REPLACE INTO watched (mid,sid) VALUES(?,?)",(mid,sid))
            else:
                cur.execute("INSERT OR REPLACE INTO watched (mid,sid) VALUES(?,?)",(mid,sid))
        return retVal
    
    def markAsWatched(self,mid="",sid=""):
        if mid == None: mid = ""
        if sid == None: sid = ""
        mid = mid.encode("utf-8")
        sid = sid.encode("utf-8")
        cur = self.getDBCursor()
        print "sid:%s, mid:%s" % (sid,mid)
        if sid != "" and mid == "":
            cur.execute("UPDATE watched SET playcount=1 WHERE sid=?",(sid,))
        elif sid != "" and mid != "":
            cur.execute("UPDATE watched SET playcount=1 WHERE sid=? AND (mid=? OR mid='')",(sid,mid))
        else:
            cur.execute("UPDATE watched SET playcount=1 WHERE mid=? AND sid=?",(mid,sid))
        
    def markAsUnWatched(self,mid="",sid=""):
        if mid == None: mid = ""
        if sid == None: sid = ""
        mid = mid.encode("utf-8")
        sid = sid.encode("utf-8")
        cur = self.getDBCursor()
        print "sid:%s, mid:%s" % (sid,mid)
        if sid != "" and mid == "":
            cur.execute("UPDATE watched SET playcount=0 WHERE sid=?",(sid,))
        elif sid != "" and mid != "":
            cur.execute("UPDATE watched SET playcount=0 WHERE sid=? AND (mid=? OR mid='')",(sid,mid))
        else:
            cur.execute("UPDATE watched SET playcount=0 WHERE mid=? AND sid=?",(mid,sid))
        
    def getWatchedEpisodes(self,sid=""):
        if sid == None: sid = ""
        sid = sid.encode("utf-8")
        cur = self.getDBCursor()
        cur.execute("SELECT count(*) FROM watched WHERE sid=? AND mid <> '' AND playcount > 0",(sid,))
        cnt = int(cur.fetchone()[0])
        return cnt
    
    def getUnWatchedEpisodes(self,sid=""):
        if sid == None: sid = ""
        sid = sid.encode("utf-8")
        cur = self.getDBCursor()
        cur.execute("SELECT count(*) FROM watched WHERE sid=? AND mid <> '' AND playcount = 0",(sid,))
        cnt = int(cur.fetchone()[0])
        return cnt