# -*- coding: utf-8 -*-
import cookielib, urllib2, zlib, socket, os, sys, time, hashlib, httplib, unicodedata, threading, re, string, copy
from cookielib import Cookie
from gzip import GzipFile
from StringIO import StringIO

try:
    import xbmc, xbmcplugin
except:
    from dev.xbmclib import *

try:
    from sqlite3 import dbapi2 as sqlite
    # print "Loading sqlite3 as DB engine"
except:
    from pysqlite2 import dbapi2 as sqlite
    # print "Loading sqlite3 as DB engine"

class IPhoenix(object):
    __cookieJar = None
    __dbfile = None
    __db = None
    __db_cursor = None
    __db_path = None
    __dbcache = None
    __defaulttimeout = None
    opener = None
    dbname = None
    dbpath = None
    cachetimeout = 3600 # default http cache 1hr
    
    def __init__(self):
        ## INFO: Need static variables here to be collactable by the GarbageCollector of Python. Otherwise this is a big Memory Leak 
        if IPhoenix.__cookieJar == None: IPhoenix.__cookieJar = self.__cookieJar = cookielib.CookieJar()
        else: self.__cookieJar = IPhoenix.__cookieJar
            
        if IPhoenix.__dbcache == None: IPhoenix.__dbcache = self.__dbcache = DBCacheHandler("HTTPDBCacheHandler.db", self.cachetimeout)
        else: self.__dbcache = IPhoenix.__dbcache
            
        if IPhoenix.opener == None:  IPhoenix.opener = self.opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(self.__cookieJar),GZipProcessor(),DeflateProcessor(),FakeBrowser(), self.__dbcache)
        else: self.opener = IPhoenix.opener

        self.opener.addheaders = [('User-Agent', 'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.7 (KHTML, like Gecko) Chrome/16.0.912.75 Safari/535.7')]
        #urllib2.install_opener(self.opener) # override default opener
        self.setTimeout()
        self.__initDatabase()
        
    def __del__(self):
        self.opener = None
        self.__cookieJar = None
        self.__dbcache = None
        if self.__db_cursor != None: 
            self.__db_cursor.close()
            self.__db_cursor = None
        if self.__db != None:
            self.__db.close()
            self.__db = None

    def __initDatabase(self):
        if self.dbname != None:
            if self.dbpath == None: self.__dbfile = xbmc.translatePath(os.path.join(xbmc.translatePath("special://database"), "phoenix" , self.dbname))
            else: self.__dbfile = xbmc.translatePath(os.path.join(self.dbpath , self.dbname))
            self.__db_path = os.path.dirname(self.__dbfile)
            if not os.path.isdir(os.path.dirname(self.__dbfile)) and self.__dbfile != ":memory:": os.makedirs(os.path.dirname(self.__dbfile))
            self.__db = sqlite.connect(self.__dbfile)
            self.__db.isolation_level = None # for autocommit
            self.__db.text_factory = str
            cursor = self.getDBCursor()
            self.initDatabase(cursor)
            del cursor

    def initDatabase(self, cursor):
        """ 
        us in combination with setting up the variable dbname to initially create tables for db 
        cursor.execute('CREATE TABLE IF NOT EXISTS seasons (season UNIQUE, contents);')
        cursor.execute('CREATE TABLE IF NOT EXISTS favorites (type, name, url, img);')
        NOTE: Method is thread save
        """
        pass
    
    def getDBCursor(self):
        if self.__db != None:
            if self.__db_cursor != None:  self.__db_cursor.close()
            self.__db_cursor = self.__db.cursor()
            self.__db_cursor.execute('PRAGMA temp_store = MEMORY;')
            self.__db_cursor.execute('PRAGMA synchronous = OFF;')
            return self.__db_cursor
        raise Exception("No Database connected. setup the dbname variable in your plugin to use a db.")
    
    def getDBConnection(self):
        if self.__db != None:
            return self.__db
        raise Exception("No Database connected. setup the dbname variable in your plugin to use a db.")
    
    def getDBPath(self):
        return self.__db_path
    
    def setTimeout(self, timeout=20):
        socket.setdefaulttimeout(timeout)
    
    def setCookies(self, cookies):
        for cookie in cookies:
            self.setCookie(cookie[0], cookie[1])
    
    def setCookie(self, name=None, value=None, cookie=None):
        """ USE to set manually a cookie for the requests either provide 
        name and value or a cookielib.Cookie objects as cookie attribute """
        if cookie != None and isinstance(cookie, Cookie):
            self.__cookieJar.set_cookie(cookie)
        elif name != None and value != None:          
            C = Cookie(version=0, name=name, value=value, port=None, port_specified=False, domain=self.domain, domain_specified=False, domain_initial_dot=False, path='/', path_specified=True, secure=False, expires=None, discard=True, comment=None, comment_url=None, rest={'HttpOnly': None}, rfc2109=False)        
            self.__cookieJar.set_cookie(C)
            
    def getCookie(self, name, default=None):
        """ get value of cookie """
        for cookie in self.__cookieJar:
            if cookie.name.lower() == name.lower():
                return cookie.value
        return default
    
    def addHeader(self, name=None, value=None):
        if name != None and value != None:
            self.addHeaders((name, value))
    
    def addHeaders(self, *args, **kwargs):
        for arg in args:
            try:
                self.opener.addheaders.append(arg)
            except:
                pass
            
    def removeHeader(self, name=None):
        if name != None and self.opener.addheaders.get(name.capitalize(), None) != None:
            del self.opener.addheaders[name.capitalize()]
            
    def getHeader(self, name, default=None):
        return self.opener.addheaders.get(name.capitalize(), default)
    
    def getUserSelection(self, slist=[], title="Choose your quality"):
        """ takes a list of strings and lets the user choose which index he wants """
        try:
            if len(slist) > 1:
                from extended.ui.SelectionDialog import SelectionDialog
                dialog = SelectionDialog()
                index = dialog.select(title, slist)
                dialog.close()
                return index
            elif len(slist) == 1:
                return 0
            else:
                return -1
        except:
            return 0
    
    def getCaptchaCode(self, captcha):
        try:
            from extended.ui.InputWindow import InputWindow
            solver = InputWindow(captcha=captcha)
            solution = solver.get()
            solver.close()
            if solution:
                return solution
            else: 
                return None
        except:
            return None
        
    def getUserInput(self):
        try:
            from extended.ui.InputWindow import InputWindow
            solver = InputWindow()
            solution = solver.get()
            solver.close()
            if solution:
                return solution
            else: 
                return None
        except:
            return None

    def showWaitDialog(self, seconds=0, title="", text="Wait %s seconds for the video stream to activate..."):
        from extended.ui.WaitDialog import WaitDialog
        dialog = WaitDialog()
        return dialog.wait(seconds, title, text)
            
    def setContent(self, content):
        """ You can use these keywords for arguments.
               content: files, songs, artists, albums, movies, tvshows, episodes, musicvideos 
               example:
                  - xbmcplugin.setContent(int(sys.argv[1]), 'movies')
        """
        # currently buggy
        #xbmcplugin.setContent(int(sys.argv[1]), 'movies')
        pass
        
    def disableHTTPCache(self):
        """ disables http chache for the next opener.open operations """
        self.setHTTPCachetimeout(0)
        
    def setHTTPCacheTimeout(self, timeout):
        """ sets http chache timeout for the next opener.open operations 
            timeout value is in sconds
        """
        if timeout == None:
            timeout = 0
        if self.__defaulttimeout == None:
            self.__defaulttimeout = self.cachetimeout
        self.__dbcache.setTimeout(timeout)
        
    def getHTTPCacheTimeout(self, timeout):
        """ returns the current value of cache expiry for the opener.open requests """
        return self.__dbcache.getTimeout(timeout)
        
    def resetHTTPCacheTimeout(self):
        """ resets the timeout for the next opener.open to its initial value """
        if self.__defaulttimeout != None:
            self.setHTTPCacheTimeout(self.__defaulttimeout)
            self.__defaulttimeout = None
        
    @staticmethod
    def log(msg, level=None):
        '''
        Writes a string to the XBMC log file. The addon name is inserted into 
        the beginning of the message automatically to help you find relevent 
        messages in the log file.
        
        The available log levels are defined in the :mod:`xbmc` module and are
        currently as follows::
        
            xbmc.LOGDEBUG = 0
            xbmc.LOGERROR = 4
            xbmc.LOGFATAL = 6
            xbmc.LOGINFO = 1
            xbmc.LOGNONE = 7
            xbmc.LOGNOTICE = 2
            xbmc.LOGSEVERE = 5
            xbmc.LOGWARNING = 3
        
        Args:
            msg (str or unicode): The message to be written to the log file.
        
        Kwargs:
            level (int): The XBMC log level to write at.
        '''
        msg = unicodedata.normalize('NFKD', unicode(str(msg))).encode('ascii', 'ignore')
        if level == None:
            level = xbmc.LOGNOTICE
        xbmc.log('%s: %s' % ('plugin.video.phoenix', msg), level)
        
    @staticmethod
    def logDebug(msg):
        IPhoenix.log(msg, xbmc.LOGDEBUG)
    
    @staticmethod
    def logError(msg):
        IPhoenix.log(msg, xbmc.LOGERROR)
    
    @staticmethod
    def logNotice(msg):
        IPhoenix.log(msg, xbmc.LOGNOTICE)
    
    @staticmethod    
    def translatePath(path):
        xbmc.translatePath(path)
        
    sceneTags = ['HHGFSKJHFUSIHFIHUSI', 'derSchreckensjaeger', 'dieKillerbakterien', 'MistgabelMoerder', 'WunschDesperate', 'x264ERFREMTiDEN', 'WELTVERBESSERER', 'BurningGreymon', 'RouletteGerman', 'VenomMyotismon', 'YoungDangerous', 'KleiderGERMAN', 'TrollololBlue', 'HUMANCENTiPAD', 'SERiENARCHiV', 'ULTiMATEx264', 'FlashForward', 'FinalRepack', 'FocusGERMAN', 'MythBusters', 'NamenGerman', 'ReplacemANT', 'SEVENTWENTY', 'seineArbeit', 'MasterClass', 'SilverHawks', 'INQUISITION', 'UNDELiCiOUS', 'DarkAvenger', 'KLASSiGERTV', 'BottleKids', 'CityCenter', 'DVDRipXviD', 'FraudulANT', 'JoLLyRoGeR', 'PokerStars', 'PrankStars', 'Prime3time', 'REPACKXviD', 'SHOWGEMiST', 'SportsDome', 'dieZukunft', 'iChristmas', 'InSecurity', 'VALiOMEDiA', 'BRICKSQUaD', 'TRANSiENCE', 'AMBASSADOR', 'MasterChef', 'COMPULSiON', 'KNiFESHARP', 'REMASTERED', 'BiTRiPPERs', 'GOREHOUNDS', 'MiNDTHEGAP', 'MOViESToRE', 'CANTonese', 'CloneWars', 'DELICIOUS', 'DEUTERIUM', 'DIEMNSION', 'FAMiLYGuY', 'INTERENAL', 'IgnorANTs', 'LateNight', 'MacDennis', 'MacGyvers', 'MacMillan', 'MauGerman', 'McClendon', 'McFarlane', 'NOsegmenT', 'POINTLESS', 'SampleFix', 'ausGerman', 'iN7TERNAL', 'iNSPiiRED', 'iStakeout', 'inDetroit', 'pHaTGiRLs', 'DEUTERiUM', 'PhoneShop', 'DEADPiXEL', 'HILSWALTB', 'McGarrett', 'ProSieben', 'REPACKERS', 'ARCHiViST', 'MovieCity', 'Ghostup10', 'LasseMaja', 'ReGenesis', 'CHRISTMAS', 'DuckTales', 'HawthoRNe', 'SpongeBob', 'PREViSiON', 'RESERVOiR', 'BLURAYRip', 'ViSiONARY', 'DIMENSION', 'GhostUp10', 'Prim3time', 'BLURAYRiP', 'iNTENTiON', 'AMBiTiOUS', 'DELiCiOUS', 'ADMiRALS', 'CartoonS', 'DEMENTED', 'DIMNSION', 'DUBBED01', 'DeLouise', 'EXPiREDD', 'FAIRPLAY', 'FLATLiNE', 'FaiLEDTV', 'LOCALNET', 'LaCarver', 'McCarthy', 'McEntire', 'McKinney', 'McPoyles', 'McQueens', 'MiRAGETV', 'N0TSC3N3', 'NLSUBBED', 'NLVLAAMS', 'NeWSTiME', 'PeepShow', 'REREPACK', 'ReVamped', 'RedBlade', 'Revoup18', 'SPRINTER', 'SeaQuest', 'ShanAucs', 'StarTrek', 'THUGLiNE', 'TRiBBLES', 'aWakeyte', 'c0nFused', 'iGNiTiON', 'iNSpiRED', 'iNTERNAC', 'iPromote', 'iReunite', 'seaQuest', 'FRAGMENT', 'McKenzie', 'McQuoddy', 'iBelieve', 'iNFiNiTE', 'SAVANNAH', 'McNamara', 'RAWNiTRO', 'LOCATiON', 'iNSPIRED', 'TELEViEW', 'BAJSKORV', 'MacGyver', 'AUDiENCE', 'HDDVDRip', 'DERANGED', 'HDDVDRiP', 'McDonald', 'ITGHATER', 'iTunesHD', 'EXTENDED', 'INSPiRED', 'STRAiGHT', 'DOCUMENT', 'SiECHTUM', 'ToONLiKE', 'MEDiEVAL', 'SPRiNTER', 'FATAL1TY', 'MONOBiLD', 'BaCKToRG', 'INTERNAL', 'MOMENTUM', 'DOLLHEAD', 'c0nFuSed', 'iNTERNAL', 'iNSPiRED', 'BASKORV', 'BaYBoYs', 'CRyPTED', 'CriMSON', 'DVDRips', 'DVDVRip', 'DePandi', 'DigiRip', 'G4YL0Rd', 'GOODBYE', 'HDTVRIP', 'LaFleur', 'MeRCuRY', 'NTERNAL', 'NinjaGo', 'OBriens', 'OMICRON', 'OnStage', 'P0W4DVD', 'PhANTom', 'ReadNFo', 'RiRAMAX', 'RiTALiN', 'StudANT', 'SubZero', 'TURKiSO', 'TVSLUTS', 'TooSoon', 'VARSiTY', 'VERSION', 'WaLMaRT', 'WuSiWuG', 'YouTube', 'derWalz', 'iNTENAL', 'iPsycho', 'iRocked', 'DVDRrip', 'FUCKING', 'HOLiDAY', 'MARiNER', 'MacLeod', 'McPhail', 'PHRENiC', 'TROLLiN', 'UNRATED', 'err0001', 'iEnrage', 'iNSPiRE', 'DogTown', 'FR34kTV', 'JFKXVID', 'JUNKies', 'JaJunge', 'SiNNERS', 'Zoey101', 'KILLERS', 'ReadNfo', 'RuPauls', 'XCOPYtv', 'M4XD0ME', 'BluByte', 'IMMERSE', 'ROUTiNE', 'ANGELiC', 'ReadNFO', 'SAPHiRE', 'DVDVRiP', 'ORPHEUS', 'MEMETiC', 'CONVERT', 'noGroup', 'MiRAMAX', 'OMiCRON', 'NODLABS', 'FR34KTV', 'DESiRED', 'TASTETV', 'UnRated', 'DiVERGE', 'READNFO', 'OldsMan', 'Numb3rs', 'MiSFiTS', 'PREMiER', 'HaNc0cK', 'CRiMSON', 'HDTVRip', 'HDTVRiP', 'EXPiRED', 'ANTics', 'AVCDVD', 'BUSTLE', 'BamVCD', 'BlueTV', 'CHRONO', 'DARNIT', 'DIRFiX', 'DVDRIp', 'DalRok', 'DejaVu', 'DiRFIX', 'DirFiX', 'FERMAN', 'FKKTVT', 'HebSub', 'HipHop', 'IMMERS', 'KlauTV', 'LaToya', 'LesBos', 'McCann', 'McCuff', 'McDuck', 'McFame', 'McLeod', 'NASCAR', 'NOTYET', 'OBrien', 'OClock', 'ONeill', 'PREAIR', 'PREAiR', 'Part10', 'QiGong', 'RENTAL', 'RTL2HD', 'S01D01', 'S01D02', 'SChiZO', 'SamSam', 'SatRip', 'ShaRes', 'TVDVDR', 'Teil10', 'UTOAiA', 'WEBRIP', 'WEBrip', 'WITSEC', 'WitSec', 'XXXVII', 'iBalls', 'iFence', 'iFight', 'iHatch', 'iHeart', 'iMight', 'iNCiTE', 'iNTENT', 'iSLAND', 'iStage', 'iStart', 'iStill', 'notITG', 'pASSed', 'AERiAL', 'ANiHLS', 'ARiSCO', 'CtrlHD', 'DirFix', 'FoVBiA', 'ICarly', 'KiDDoS', 'McBeal', 'PSYCHD', 'REALLY', 'REFiLL', 'RePack', 'SAT1HD', 'SUBFIX', 'TalANT', 'TinTin', 'WATERS', 'XTERRA', 'XtremE', 'iBloop', 'iSaved', 'iSpace', 'iThink', 'iTwins', 'notNVA', 'DVDiSO', 'NATIVE', 'ORENJi', 'REPOST', 'RETAIL', 'DiRFiX', 'RETAiL', 'SUBFiX', 'WebRip', 'FaiLED', 'ORENJI', 'WEBRiP', 'iCarly', 'BLOWME', 'DEMAND', 'GNARLY', 'GOTHiC', 'OSiRiS', 'STUDIO', 'iNTERN', 'DIRFIX', 'DVDSCR', 'SyntaX', 'kidsTV', 'WEBRip', 'EXTRAS', 'RELiVE', 'piepHD', 'ANALTV', 'HAGGiS', 'HTVRiP', 'FFNDVD', 'WOMBAT', 'iNFOTv', 'RUNNER', 'DEiMOS', 'DiETER', 'PiXELS', 'SiMPTY', 'ARCHIV', 'Repack', 'REWARD', 'PL4SM4', 'SUBBED', 'german', 'NMBSTV', 'JuniOr', 'sub by', 'PUNiCA', 'CLERKS', 'DVDRIP', 'DVDrip', 'ARCHiV', 'SATRiP', 'BLURAY', 'Custom', 'PROPER', 'Subbed', 'REPACK', 'SAiNTS', 'BluRay', 'SATRip', 'UTOPiA', 'Dubbed', 'DUBBED', 'DVDRiP', 'GERMAN', 'DVDRip', 'German', 'BOLOX', 'BRAVO', 'BSGTV', 'CHGRP', 'CaRaT', 'DANKE', 'DVDip', 'DVRiP', 'DeRay', 'EMail', 'EXCEL', 'FIXED', 'FiNAL', 'GOTEi', 'HUERS', 'KITTs', 'KhaRi', 'KiNGS', 'LMFAO', 'McCox', 'McCoy', 'McKay', 'McKee', 'NCAXA', 'OHara', 'OPTIK', 'PiXTV', 'RIVER', 'SAiNT', 'SEALd', 'SUPER', 'TASTE', 'Tag10', 'Tag11', 'Tag12', 'Tag13', 'TibaK', 'VoDHD', 'VoMiT', 'ZSiSO', 'iCook', 'iCrap', 'iDont', 'iFind', 'iGNTV', 'iGive', 'iHate', 'iHurt', 'iKiss', 'iLost', 'iLove', 'iMake', 'iMust', 'iSams', 'iSell', 'iTake', 'iWant', 'mVzTV', 'AHDTV', 'CENTi', 'CHiPs', 'DSRip', 'DUKES', 'EXViD', 'FINAL', 'GLOBO', 'HNRTV', 'SEELE', 'ViPER', 'WASTE', 'iDate', 'iHave', 'iLook', 'iMeet', 'iMove', 'iQuit', 'iWont', 'McGee', 'TVRiP', 'TVRip', 'HAPPY', 'PHASE', 'BALKO', 'FEVER', 'SAMFD', 'FiXTv', 'RERiP', 'MACRO', 'DVSKY', 'dEVEx', 'DAONC', 'RERIP', 'iNGOT', 'HODEN', 'OSiTV', 'FiHTV', 'GREEK', 'LiGHT', 'sHoTV', 'H 264', 'TOPAZ', 'SHDTV', 'BARGE', 'CRNTV', 'ETMID', 'ETACH', 'RiVER', 'RETRO', 'MERRY', 'HDDVD', 'YesTV', 'BDRiP', 'GM4FS', 'SiGHT', 'SiTiN', 'ANIME', 'UNCUT', 'WebHD', 'BDRip', 'STARS', 'CRiSP', 'ANiME', 'ZZGtv', 'aWake', 'FKKTV', '720P', 'ACME', 'ANAX', 'AWOL', 'C16s', 'CABS', 'COHD', 'COPS', 'DONT', 'DiME', 'DiRK', 'EAST', 'EPiC', 'ESAB', 'FAiL', 'GORE', 'HTDV', 'HiQT', 'KJEM', 'KUSH', 'LEGO', 'LENA', 'LOKi', 'MEGA', 'MOAB', 'MiNT', 'NCIS', 'NERV', 'OMAC', 'PDTv', 'PIEP', 'PJem', 'RNFO', 'S03c', 'SPQR', 'SSTV', 'SSWR', 'STaN', 'TAPA', 'TRAM', 'TRIC', 'USOS', 'XXXV', 'XivD', 'XvID', 'YMCA', 'YooX', 'aNBc', 'aWak', 'iGet', 'iGot', 'iOwe', 'iPie', 'iSaw', 'iToe', 'iVTV', 'ABCs', 'ABFs', 'GRiM', 'MOTU', 'NFHD', 'SiTV', 'VIII', 'WASP', 'WWII', 'grAS', 'grAs', 'iFix', 'iWas', 'w0rm', 'DOCU', 'GoTV', 'SiSO', 'iWin', 'CoCo', 'HDCP', 'NoTV', 'SiNK', 'URTV', 'VFUA', 'XTVC', 'YEAR', 'NASA', 'ViLD', 'WRCR', 'WiDE', 'XDiD', 'd0ne', 'FUtV', 'SQsR', 'AC3D', 'P0W4', 'SLOW', 'XMAS', 'NOVA', 'CLUE', 'DiVX', 'BiTW', 'HP4F', 'ASAP', 'Xvid', 'TVEP', 'NYPD', 'HACO', 'DivX', 'ILVS', 'SMEG', 'C4TV', 'X264', 'LMAO', 'REAL', 'READ', 'iFPD', 'XVID', 'SOKO', 'CABs', 'CRoW', 'TMSF', 'DOKU', 'h264', 'PDTV', 'euHD', 'idTV', 'TCPA', 'xviD', 'GTVG', 'XViD', 'x264', '720p', 'HDTV', 'XviD', 'ABC', 'AFG', 'AMG', 'BAX', 'BBC', 'BBQ', 'BFJ', 'BOV', 'BSN', 'BTN', 'BiQ', 'CJs', 'CON', 'D4U', 'DDR', 'DNR', 'DSF', 'DSV', 'DSr', 'DTV', 'DWB', 'EMP', 'FFA', 'FKK', 'FOF', 'FPM', 'FUA', 'GDR', 'GoT', 'HDV', 'HIV', 'HSV', 'KFV', 'KGB', 'KiK', 'MAS', 'MCW', 'MIG', 'MTA', 'MTV', 'MVP', 'MaM', 'MaX', 'NEN', 'NHB', 'NRL', 'OGD', 'OMP', 'P3X', 'P4X', 'PJD', 'PJs', 'POV', 'PVP', 'PhD', 'RIP', 'RJR', 'RJs', 'RPA', 'RTV', 'SFK', 'SNU', 'SOL', 'SVA', 'SnO', 'TAS', 'TBs', 'TCM', 'TEE', 'TGF', 'TKO', 'ToF', 'UNi', 'USS', 'V4F', 'VEW', 'VFR', 'WMZ', 'WTF', 'XMC', 'XTV', 'XVI', 'ZiG', 'aWa', 'iDo', 'iGo', 'iHT', 'iKA', 'iNK', 'iTV', 'uDF', 'vRs', 'w4f', 'ASA', 'DIY', 'DMV', 'DOT', 'FOV', 'GDA', 'GTV', 'JFK', 'KFC', 'MAD', 'TBS', 'TGS', 'THC', 'TIK', 'TTK', 'TtC', 'XII', 'dGC', 'iFH', 'AVS', 'CIA', 'CTU', 'DPC', 'GSI', 'H8R', 'HRG', 'LSD', 'OCC', 'OTT', 'PEZ', 'PTV', 'PVR', 'REA', 'SUV', 'TVi', 'USA', 'ARC', 'BFF', 'III', 'MDX', 'NEW', 'P2P', 'RAK', 'SMA', 'TVR', 'VIP', 'kHz', 'DNS', 'GeT', 'RTL', 'XOR', 'FDL', 'NYC', 'AEN', 'ATG', 'FoV', 'SOS', 'WAT', 'KYR', 'PFa', 'SYA', 'dVe', 'BoU', 'FBI', 'CSI', 'UFO', 'IRT', 'SYS', 'J4F', 'RTA', 'TvR', 'BiA', 'DD5', 'KDD', 'DAH', 'LoD', 'INT', 'TWG', 'WEB', 'MTS', 'GEO', 'MiG', 'iNF', 'MFi', 'TVS', 'TVH', 'SVU', 'GVD', 'NTB', 'QPS', 'IND', 'WPi', 'TLA', 'DVB', 'W4F', 'FTP', 'UMF', 'AMC', 'CIS', 'BWB', 'SFM', 'GMP', 'MFG', 'LOL', 'SOV', 'TOS', 'H2O', 'DHD', 'QCF', 'DSR', 'PSH', 'PkZ', 'NNS', 'ALF', 'NGE', 'DAW', 'CiD', 'iND', 'TNG', 'ETM', 'BSR', 'MWS', 'NFO', 'aAF', '2HD', 'CDD', 'CiA', 'NMP', 'MSE', 'QoM', 'OHD', 'NVA', 'iNT', 'RSG', 'FQM', 'ITG', 'SOF', 'TVP', 'dTV', 'AC', 'AG', 'AJ', 'AM', 'BH', 'CQ', 'DB', 'DC', 'DN', 'EL', 'FC', 'GC', 'GE', 'GX', 'HD', 'HP', 'IL', 'IQ', 'MD', 'MY', 'NS', 'OF', 'OH', 'PR', 'PS', 'QI', 'SP', 'TB', 'TL', 'TR', 'VI', 'VS', 'XL', 'XX', 'iN', 'iQ', 'xV', 'AR', 'CA', 'DJ', 'GT', 'IV', 'MS', 'NC', 'NT', 'SD', 'SJ', 'TX', 'US', 'UT', 'WG', 'JJ', 'LA', 'OO', 'YT', 'SF', 'AK', 'HR', 'TS', 'DS', 'KW', 'BD', 'ER', 'XG', 'OP', 'RJ', 'SK', 'NZ', 'RC', 'AU', 'NY', 'IT', 'SG', 'XY', 'CI', 'II', 'IS', 'BE', 'TV', 'UK', 'HS', 'TU', 'VF', 'DL', 'FS', 'WS']
    titleReplacemants = [
                            ("\xc3\xbc","ü"),
                            ("\xc3\xb6","ö"),
                            ("\xc3\xa4","ä"),
                            ("&amp;","&")
                         ] 

    @staticmethod
    def __cleanPart(part):
        try:
            part = part.encode('utf-8','replace')
        except:
            pass
        try:
            part = re.sub("(\(\s*[1|2]{1}[0-9]{3}\s*\))", "", part) # remove years with ()
            part = re.sub("(\s*[1|2]{1}[0-9]{3}\s*)", "", part) # remove years
            part = re.sub("(\([^\)]{2,}\))", "", part) # replace all (xxxx)
            part = re.sub("(\<[^\>]+\>)", "", part) # replace html tags
            for s in IPhoenix.titleReplacemants: part = part.replace(s[0],s[1])
            for tag in IPhoenix.sceneTags: part = part.replace(tag,"")
            for c in "!\"#$%&'()*+,-./;<=>?@[\\]^_`{|}~": part=part.replace(c," ") # remove punctuation
            part = re.sub("(\s+)", " ", part) # replace double spaces
            part = part.strip()
        except:
            pass
        return part
    
    
    @staticmethod
    def cleanSceneTitle(title):
        try:
            title = title.encode('utf-8','replace')
        except:
            pass
        title = title.replace("\r","").replace("\n","").replace("."," ").strip()
        try:
            f = re.split("(S[0-9]+E[E|0-9]+)",title,re.I)
            if f != None and len(f) == 3:
                prefix = IPhoenix.getEpisodeTitlePrefix(f[1])
                if prefix != None: 
                    postfix = IPhoenix.__cleanPart(f[2])
                    if postfix != "" and not prefix in postfix:
                        title = prefix + ": " + postfix
                    else:
                        title = prefix
                else: 
                    title = IPhoenix.__cleanPart(f[2])
            else:
                f = re.split("(E[E|0-9]+)",title,re.I)
                if f != None and len(f) == 3:
                    prefix = IPhoenix.getEpisodeTitlePrefix(f[1])
                    if prefix != None:
                        postfix = IPhoenix.__cleanPart(f[2])
                        if postfix != "" and not prefix in postfix:
                            title = prefix + ": " + postfix
                        else:
                            title = prefix
                    else: 
                        title = IPhoenix.__cleanPart(f[2])
                else: 
                    title = IPhoenix.__cleanPart(title)
        except:
            pass
        return title
    
    @staticmethod
    def getEpisodeNumber(title):
        try:
            title = title.encode('utf-8','replace')
        except:
            pass
        title = title.replace("\r","").replace("\n","").replace("."," ").strip()
        try:
            f = re.search("S[0-9]+E([0-9]+)",title,re.I)
            if f and len(f.groups()) > 0:
                return int(f.group(1))
            else:
                f = re.search("E([0-9]+)",title,re.I)
                if f and len(f.groups()) > 0:
                    return int(f.group(1))
        except:
            pass
        return -1
    
    @staticmethod
    def getEpisodeTitlePrefix(title):
        try:
            title = title.encode('utf-8','replace')
        except:
            pass
        title = title.replace("\r","").replace("\n","").replace("."," ").strip()
        try:
            f = re.search("E([E|0-9]+)",title,re.I)
            if f and len(f.groups()) > 0:
                prefix = "Episode "
                if re.match("^[0-9]+$",f.group(1)):
                    prefix = prefix+"%02d" % int(f.group(1))
                    return prefix
                else:
                    f = re.search("([0-9]+)e([0-9]+)",f.group(1),re.I)
                    if f and len(f.groups()) == 2:
                        prefix = prefix+"%02d %02d" % (int(f.group(1)),int(f.group(2)))
                        return prefix
        except:
            pass
        return None
    
    @staticmethod
    def getSeasonNumber(title):
        try:
            title = title.encode('utf-8','replace')
        except:
            pass
        title = title.replace("\r","").replace("\n","").replace("."," ").strip()
        try:
            f = re.search("S([0-9]+)E[E|0-9]+",title,re.I)
            if f and len(f.groups()) > 0:
                return int(f.group(1))
        except:
            pass
        return -1
    
    @staticmethod
    def getYearFromTitle(title):
        try:
            title = title.encode('utf-8','replace')
        except:
            pass
        title = title.replace("\r","").replace("\n","").replace("."," ").strip()
        f = re.search("\(\s*([1|2]{1}[0-9]{3})\s*\)",title)
        if f and len(f.groups()) > 0 and re.match("[0-9]+",f.group(1)):
            return int(f.group(1))
        else:
            f = re.search("\s*([1|2]{1}[0-9]{3})\s*",title)
            if f and len(f.groups()) > 0 and re.match("[0-9]+",f.group(1)):
                return int(f.group(1))
        return 0
    
    @staticmethod
    def dictMerge(a, b):
        if a is None: return b
        if b is None: return a
        if isinstance(a, list) and isinstance(b, list):
            return list(set(a + b))
        elif isinstance(a, set) and isinstance(b, set):
            return set(list(a) + list(b))
        elif isinstance(a, dict) and isinstance(b, dict):
            # assume d1 and d2 are dictionaries
            keys = set(a.iterkeys()) | set(b.iterkeys())
            return dict((key, IPhoenix.dictMerge(a.get(key), b.get(key))) for key in keys)
        else:
            if b.strip() != "": return b
            return a
            
    
    """### HELPER FUNCTIONALITY END """
    
class DeflateProcessor(urllib2.BaseHandler):
    """A handler to add deflate capabilities to urllib2 requests
    """
    def http_request(self, req):
        if req.headers.get("Accept-encoding", None) == None:
            req.add_header("Accept-encoding", "deflate")
        else:
            req.headers["Accept-encoding"] = req.headers.get("Accept-encoding") + ",deflate"
        return req
    https_request = http_request
    
    def deflate(self, data):   # zlib only provides the zlib compress format, not the deflate format;
        try:               # so on top of all there's this workaround:
            return zlib.decompress(data, -zlib.MAX_WBITS)
        except zlib.error:
            return zlib.decompress(data)

    def http_response(self, req, resp):
        if resp.headers.get("content-encoding") == "deflate":
            gz = StringIO(self.deflate(resp.read()))
            old_resp = resp
            del old_resp.headers['content-encoding']
            resp = urllib2.addinfourl(gz, old_resp.headers, old_resp.url, old_resp.code)  # 'class to add info() and
            resp.msg = old_resp.msg
        return resp
    https_response = http_response



class GZipProcessor(urllib2.BaseHandler):
    """A handler to add gzip capabilities to urllib2 requests
    """
    def http_request(self, req):
        if req.headers.get("Accept-encoding", None) == None:
            req.add_header("Accept-encoding", "gzip")
        else:
            req.headers["Accept-encoding"] = req.headers.get("Accept-encoding") + ",gzip"
        return req
    https_request = http_request

    def http_response(self, req, resp):
        if resp.headers.get("content-encoding") == "gzip":
            gz = GzipFile(fileobj=StringIO(resp.read()), mode="r")
            old_resp = resp
            del old_resp.headers['content-encoding']
            resp = urllib2.addinfourl(gz, old_resp.headers, old_resp.url, old_resp.code)
            resp.msg = old_resp.msg
        return resp
    https_response = http_response
        
    
    

class FakeBrowser(urllib2.BaseHandler):
    referer = "http://www.google.com"
    """Fake being a normal Browser
    """
    def http_request(self, req):
        if req.headers.get("Accept", None) == None:
            req.add_header("Accept", 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8')
        if req.headers.get("Accept-charset", None) == None:
            req.add_header("Accept-Charset", 'ISO-8859-1,utf-8;q=0.7,*;q=0.3')
        if req.headers.get("Accept-language", None) == None:    
            req.add_header("Accept-Language", 'en-US;q=0.6,en;q=0.4')
        if req.headers.get("Cache-control", None) == None:
            req.add_header("Cache-Control", 'max-age=0')
        if req.headers.get("Connection", None) == None:
            req.add_header("Connection", 'keep-alive')
        # Auto handling of referer
        if req.headers.get("Referer", None) == None:
            req.add_header("Referer", str(self.referer))
        self.referer = req.get_full_url()
        #req.add_header("Host", req.get_host())
        #req.add_header("Referer", req.get_full_url())
        #print "###############################"
        #print randUA
        #print req.header_items()
        #print req.get_type()
        #print req.get_full_url()
        #print "###############################"
        return req
    https_request = http_request
    
class ThrottlingProcessor(urllib2.BaseHandler):
    """Prevents overloading the remote web server by delaying requests.

    Causes subsequent requests to the same web server to be delayed
    a specific amount of seconds. The first request to the server
    always gets made immediately"""
    __shared_state = {}
    def __init__(self, throttleDelay=5):
        """The number of seconds to wait between subsequent requests"""
        # Using the Borg design pattern to achieve shared state
        # between object instances:
        self.__dict__ = self.__shared_state
        self.throttleDelay = throttleDelay
        if not hasattr(self, 'lastRequestTime'):
            self.lastRequestTime = {}
        
    def default_open(self, request):
        currentTime = time.time()
        if ((request.host in self.lastRequestTime) and
            (time.time() - self.lastRequestTime[request.host] < self.throttleDelay)):
            self.throttleTime = (self.throttleDelay - (currentTime - self.lastRequestTime[request.host]))
            # print "ThrottlingProcessor: Sleeping for %s seconds" % self.throttleTime
            time.sleep(self.throttleTime)
        self.lastRequestTime[request.host] = currentTime
        return None

    def http_response(self, request, response):
        if hasattr(self, 'throttleTime'):
            response.info().addheader("x-throttling", "%s seconds" % self.throttleTime)
            del(self.throttleTime)
        return response
    

class DBCacheHandler(urllib2.BaseHandler):
    __dbfile = ":memory:"
    __cachetime = 3600
    __db = None
    
    def __init__(self, dbname=":memory:", cachetime=3600):
        """The location of the cache directory"""
        self.__cachetime = cachetime
        if cachetime > 0:
            self.__dbfile = xbmc.translatePath(os.path.join(xbmc.translatePath("special://database"), os.path.join("phoenix", "core", dbname)))
            if not os.path.isdir(os.path.dirname(self.__dbfile)) and self.__dbfile != ":memory:":
                    os.makedirs(os.path.dirname(self.__dbfile))
            self.__db = sqlite.connect(self.__dbfile)
            self.__db.isolation_level = None # for autocommit
            self.__db.text_factory = str
            cursor = self.__db.cursor()
            cursor.execute('PRAGMA temp_store = MEMORY;')
            cursor.execute('PRAGMA synchronous = OFF;')
            cursor.execute('CREATE TABLE IF NOT EXISTS cache (hashkey UNIQUE, timestamp INTEGER, timeout INTEGER,status_code INTEGER, header, body);')
            cursor.execute('CREATE TABLE IF NOT EXISTS last_cleanup (timestamp INTEGER);')
            # delete cache oder than 7 days
            #cursor.execute('DELETE FROM cache WHERE timestamp < ?',((int(round(time.time()))-604800),))
            
            cleanup = False
            now = int(round(time.time()))
            cursor.execute('SELECT timestamp from last_cleanup')
            last_cleanup = cursor.fetchone()
            if (last_cleanup == None) or (last_cleanup != None and (now - int(last_cleanup[0])) > 604800): # autoclean every 7 days
                cleanup = True
            #elif last_cleanup != None:
            #    IPhoenix.logNotice(">>>>> HTTPCACHE: next cleanup in %s sec." % (604800-((now-int(last_cleanup[0])))));
            if cleanup == True:
                IPhoenix.logDebug(">>>>> HTTPCACHE: cleaning cache.");
                cursor.execute('DELETE FROM last_cleanup')
                cursor.execute('INSERT INTO last_cleanup (timestamp) VALUES(?)', (now,))
                cursor.execute('DELETE FROM cache WHERE ROUND(timestamp-?) > timeout', (now,))
                if hasattr(cursor, 'rowcount'):
                    IPhoenix.logDebug(">>>>> HTTPCACHE: removed %s timedout entries." % cursor.rowcount);
            cursor.close()
            
    def __del__(self):
        if self.__db != None:
            try:
                self.__db.close()
            except:
                pass
        self.close()
            
    def getDBConnection(self):
        if self.__db != None:    
            return self.__db
        return None 
        
    def setTimeout(self, timeout=3600):
        IPhoenix.logDebug(">>>>> HTTPCACHE timedout set to: %s sec" % timeout);
        self.__cachetime = timeout
        
    def getTimeout(self):
        return self.__cachetime
        
    def default_open(self, request):
        if (self.__cachetime > 0 and (request.get_method() == "GET") and DBCachedResponse.existsInCache(self.__db, request.get_full_url())):
            return DBCachedResponse(self.__db, request.get_full_url(), setCacheHeader=True)
        return None # let the next handler try to handle the request
    
    def http_response(self, request, response):
        headers = response.info()
        contentType = str(headers.get("Content-type")).lower()
        #print ">>>>>>>>>>>>>> Content-Type: %s" % contentType
        if request.get_method() == "GET" and self.__cachetime > 0 and (contentType.startswith("text/") or "/json" in contentType or "/xml" in contentType or "/xhtml" in contentType):
            if not 'DBCachedResponse' in str(response.info()):
                DBCachedResponse.storeInCache(self.__db, request.get_full_url(), response, self.__cachetime)
                return DBCachedResponse(self.__db, request.get_full_url(), setCacheHeader=False)
            else:
                return DBCachedResponse(self.__db, request.get_full_url(), setCacheHeader=True)
        elif not 'DBCachedResponse' in str(response.info()):
            pass
            # uncachable
        return response
        
class DBCachedResponse(StringIO):
    """An urllib2.response-like object for cached responses.
    To determine wheter a response is cached or coming directly from
    the network, check the x-cache header rather than the object type."""
    
    @staticmethod
    def existsInCache(db, url, timeout=None):
        if timeout == 0:
            return False
        retVal = False
        hashkey = hashlib.md5(url).hexdigest()
        cur = db.cursor()
        cur.execute("SELECT timestamp, timeout FROM cache WHERE hashkey=?", (hashkey,))
        res = cur.fetchone()
        retVal = (res != None)
        if retVal:
            cached = int(res[0])
            if timeout == None: # if no extra timeout is set use saved
                timeout = int(res[1])
            now = int(round(time.time()))
            retVal = (now - cached <= timeout)
            if retVal == False:
                cur.execute("DELETE FROM cache WHERE hashkey=?", (hashkey,))
                IPhoenix.logDebug(">>>>> HTTPCACHE timedout!");
            else:
                IPhoenix.logDebug(">>>>> HTTPCACHE HIT: %s, hash:%s, timeout in: %s sec" % (retVal, hashkey, (timeout - (now - cached))));
        cur.close()
        return retVal;

    @staticmethod
    def storeInCache(db, url, response, timeout):
        if timeout > 0:
            IPhoenix.logDebug(">>>>> HTTPCHACHE Saved: %s" % url)
            hashkey = hashlib.md5(url).hexdigest()
            cur = db.cursor()
            cur.execute('PRAGMA temp_store = MEMORY;')
            cur.execute('PRAGMA synchronous = OFF;')
            cur.execute("INSERT or REPLACE into cache (hashkey,timestamp,timeout,status_code,header,body) VALUES(?,?,?,?,?,?)", (hashkey, int(round(time.time())), timeout, response.code , zlib.compress(str(response.info())).encode('base64', 'strict'), zlib.compress(response.read()).encode('base64', 'strict')))
            cur.close()
            
    
    def __init__(self, db, url, setCacheHeader=True):
        hashkey = hashlib.md5(url).hexdigest()
        
        cur = db.cursor()
        cur.execute('PRAGMA temp_store = MEMORY;')
        cur.execute('PRAGMA synchronous = OFF;')
        cur.execute("SELECT status_code,header,body FROM cache WHERE hashkey=?", (hashkey,))
        data = cur.fetchone()
        cur.close()
        if data == None:
            IPhoenix.logError(">>>>> HTTPCHACHE : No Data returned for url:%s hashkey:%s" % (url,hashkey))
            raise Exception()
        try:
            StringIO.__init__(self, zlib.decompress(data[2].decode('base64', 'strict')))
        except MemoryError, e:
            IPhoenix.logError("%s" % e)
            cur = db.cursor()
            cur.execute("DELETE FROM cache WHERE hashkey=?", (hashkey,)) # remove to try fetch again next time
            cur.close()
            StringIO.__init__(self,"")
        
        self.url = url
        self.code = int(data[0])
        self.msg = "OK"
        headerbuf = zlib.decompress(data[1].decode('base64', 'strict'))
        if setCacheHeader:
            headerbuf += "x-cache: DBCachedResponse/%s\r\n" % (hashkey)
        self.headers = httplib.HTTPMessage(StringIO(headerbuf))

    def info(self):
        return self.headers
    def geturl(self):
        return self.url
