import sys, os
from lib import IPhoenix
try:
    import xbmc, xbmcgui
except:
    from dev.xbmclib import *

class Plugins:
    __baseDir = None
    __loadedPlugins = {}
    __currentPlugin = None
    
    def __init__(self,baseDir):
        self.__baseDir = baseDir
    
    def getDirectory(self,path="",args={}):
        dialog = xbmcgui.DialogProgress()
        categoryList = []
        if path == "":
            dialog.create('Loading Directory', 'loading directory...')
            dialog.update(0)
            # main index list plugin types
            categoryList = [ {"path":"video","title":"Video","thumbnail":"videosfolder","isFolder":True} ]
        elif not "/" in path:
            dialog.create('Loading Directory', 'loading directory...')
            dialog.update(0)
            dirs = os.listdir(os.path.join(self.__baseDir,path))
            cntdirs = len(dirs)
            cnt = 0
            # list plugins in category
            for name in dirs:
                cnt = cnt+1
                if os.path.isdir(os.path.join(os.path.join(self.__baseDir,path), name)) and not name.startswith("."):
                    try:
                        dialog.update(int((cnt/float(cntdirs/100.0))))
                        plugin = self.loadPlugin(name,path)
                        categoryList.append({ "path":"%s/%s"%(path,name),"title": plugin.getTitle(), "thumbnailImage":plugin.getThumbnail(), "isFolder":True, "type":path,"infoLabels":{"backdrop_url":plugin.getFanArt()}, "args":{}})
                    except:
                        #raise
                        pass
        elif "/" in path:
            dialog.create('Loading Metadata', 'loading metadata...')
            dialog.update(0)
            #entered plugin call plugin internal getCategory
            plugindirectory = path.split("/")
            # remove main categories eg. video/pluginname
            path = plugindirectory.pop(0)
            name = plugindirectory.pop(0)
            plugin = self.loadPlugin(name,path);
            if plugin != None:
                categoryList = self.processPluginResponse(path,name,plugin,plugindirectory,dialog,plugin.getDirectory("/".join(plugindirectory),args))
        dialog.close()
        return categoryList
    
    def searchPlugin(self,path,searchword):
        dialog = xbmcgui.DialogProgress()
        dialog.create('Searching', 'searching...')
        dialog.update(0)
        plugindirectory = path.split("/")
        path = plugindirectory.pop(0)
        name = plugindirectory.pop(0)
        plugin = self.loadPlugin(name,path);
        if plugin != None and plugin.isSearchable:
            categoryList = self.processPluginResponse(path,name,plugin,plugindirectory,dialog,plugin.search(searchword))
        dialog.close()
        return categoryList
        
    
    def processPluginResponse(self,path,name,plugin,plugindirectory,dialog,categoryList):
        from metadata import MetaData
        meta = MetaData()
        pluginDefaults = self.getPluginDefaults(plugin)
        _catLen = len(categoryList)
        _cnt = 0
        lastPerc = -1
        enrichedCategoryList = []
        for category in categoryList:
            _cnt = _cnt + 1
            perc = int(_cnt/(_catLen/100.0))
            if perc == 0: perc = 1
            if lastPerc != perc or lastPerc == -1:
                dialog.update(perc, "Title:  %s" % IPhoenix.cleanSceneTitle(category.get("title","No Title")),"","loaded %d of %d" %(_cnt,_catLen))
            lastPerc = perc
            if dialog.iscanceled() == True: break
            if category.get("path",None) != None:
                cleanPath = ""
                if not category["path"].startswith("/"):
                    cleanPath = ("%s/%s"%("/".join(plugindirectory),category["path"]))
                    if cleanPath.endswith("/"):
                        cleanPath = cleanPath[:-1]
                else:
                    cleanPath = category["path"][1:]
                category["path"]="%s/%s/%s" %(path,name,cleanPath)
                category["path"] = category["path"].replace("//","/")
            else:
                cleanPath = "/".join(plugindirectory)
                if cleanPath.endswith("/"):
                    cleanPath = cleanPath[:-1]
                category["path"]="%s/%s/%s" %(path,name,cleanPath)
                category["path"] = category["path"].replace("//","/")
            if category.get("type",None) == None:
                category["type"]=path.lower()
            else:
                if category.get("type") in (MetaData.typeMovie,MetaData.typeTVShow):
                    title = IPhoenix.cleanSceneTitle(category.get("title",""))
                    year = category.get("year",None) 
                    if year == None or year==0: year = IPhoenix.getYearFromTitle(category.get("title",""))
                    category["infoLabels"] = IPhoenix.dictMerge(category.get("infoLabels",{}),meta.getMeta(category.get("type"), title,category.get("imdb_id",""), category.get("tmdb_id",""), year))
                elif category.get("type") == MetaData.typeSeason:
                    title = IPhoenix.cleanSceneTitle(category.get("title",""))
                    season = category.get("season",-1)
                    if season == -1: season = IPhoenix.getSeasonNumber(category.get("title",""))
                    seasons = meta.getSeasons(title, category.get("imdb_id",""), season)
                    if len(seasons) > 0: 
                        category["infoLabels"] = IPhoenix.dictMerge(category.get("infoLabels",{}),seasons[0])
                elif category.get("type") == MetaData.typeEpisode:
                    title = IPhoenix.cleanSceneTitle(category.get("title","")) 
                    season = category.get("season",-1)
                    if season == -1: season = IPhoenix.getSeasonNumber(category.get("title",""))
                    episode = category.get("episode",-1)
                    if episode == -1: episode = IPhoenix.getEpisodeNumber(category.get("title",""))
                    imdb_id = category.get("imdb_id","")
                    year = IPhoenix.getYearFromTitle(title)
                    if year > 0: title = "%s (%s)" % (title,year)
                    if imdb_id == "":
                        seasons = meta.getSeasons(title, "", season)
                        if len(seasons) > 0: imdb_id = seasons[0].get("imdb_id","")
                    category["infoLabels"] = IPhoenix.dictMerge(category.get("infoLabels",{}),meta.getEpisodeMeta(title, imdb_id, season, episode))
                category["title"] = category.get("infoLabels",{}).get("title",title)
            enrichedCategoryList.append(IPhoenix.dictMerge(pluginDefaults, category))
        return enrichedCategoryList
        
    
    def getPluginDefaults(self,plugin=None):
        return {
               "infoLabels":{ "backdrop_url":plugin.getFanArt() },
               "thumbnailImage":plugin.getThumbnail(),
               "iconImage" : plugin.getThumbnail(),
               "type":"video"
        }
    
    def getPlugin(self,path=None):
        if self.__currentPlugin == None and path != None and "/" in path:
            plugindirectory = path.split("/")
            path = plugindirectory.pop(0)
            name = plugindirectory.pop(0)
            self.loadPlugin(name,path);
        return self.__currentPlugin
    
    def loadPlugin(self,name,path=None):
        plugin = self.__loadedPlugins.get(name,None)
        if plugin == None and path != None:
            try:
                # from plugins.video import serienjunkies 
                istr = str("plugins.%s.%s"%(path,name))
                frlst = str("plugins.%s"%path)
                mod = __import__(istr, fromlist=[frlst])
                obj = getattr(mod, path.capitalize())
                self.__loadedPlugins[name] = obj(homePath=xbmc.translatePath(os.path.join(self.__baseDir,path,name)))
                self.__currentPlugin = self.__loadedPlugins[name]
                return self.__currentPlugin
            except:
                raise
                return None
        else:
            return plugin
    