from plugins.video import IVideo
from lib.BeautifulSoup import BeautifulSoup
from metadata import MetaData
import re, time, json, urllib
from lib import IPhoenix

class Video(IVideo):
    title = "serienjunkies.org"
    dbname = "serienjunkies.db"
    isSearchable = True
    
    AZ_DIRECTORIES = ['#','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y', 'Z']
    BASE_URL = 'http://serienjunkies.org'
    
    def initDatabase(self, cursor):
        IVideo.initDatabase(self, cursor)
        cursor.execute('CREATE TABLE IF NOT EXISTS streamids (ids VARCHAR(25) UNIQUE,url VARCHAR(4096) UNIQUE);')
        cursor.execute('CREATE TABLE IF NOT EXISTS series (sid VARCHAR(25) UNIQUE, title VARCHAR(25), url VARCHAR(4096) UNIQUE);')
        cursor.execute("CREATE INDEX IF NOT EXISTS idx_series_url on series (url);")
    
    def getDirectory(self,path,args={}):
        self.setContent("tvshows")
        if path == "":
            return [
                {"path":"/updates","title":"Series Updates","args":{"series":True}},
                {"path":"/updates","title":"Episode Updates","args":{"episodes":True}},
                {"path":"/a-z","title":"A-Z"}
            ]
        elif path == "updates":
            if args.get("series",False) == True:
                return self.getSeriesUpdates()
            elif args.get("episodes",False) == True:
                return self.getEpisodeUpdates()
        elif path.startswith("a-z"):
            return self.getAZList(args.get("letter",None))
        elif path == "search":
            pass
        elif path == "seriesdetail":
            return self.getSeriesDetail(args.get("url",None))
        elif path == "seasondetails":
            return self.getSeasonDetail(args.get("url",None),args.get("season",None),args.get("episode",None))
        elif path == "hosterselection":
            return self.getHoserSelection(args.get("hoster",None),args.get("sid",None),args.get("mid",None))
        return []
    
    def getHoserSelection(self,hoster,sid,mid):
        retList = []
        if hoster != None:
            for key in hoster.iterkeys():
                retList.append({"title":key,"mid":mid,"sid":sid,"showWatched":False,"play":{"url":"http://crypt2.net/file/%s"%hoster.get(key)}})
        return retList;
    
    def getSeasonDetail(self,url,season=-1,episode=-1):
        retList = []
        ids = self.getIds(url)
        sid = self.getSidByUrl(url)
        if ids != None:
            res = self.opener.open("http://serienjunkies.org/media/ajax/getStreams.php?s=%s&ids=%s"%(season,ids))
            jsonString = res.read()
            streams = json.loads(jsonString)
            eplist = []
            for stream in streams:
                #id = stream[0]
                title = stream[1]
                language = stream[2]
                if language == "german" or language == "ger" or language == "d" or language == "dl": language = "[DE]"
                elif language == "english" or language == "eng" or language == "e": language = "[EN]"
                hosters = stream[4]
                hd = (stream[5] == 1)
                quality = "[SD]"
                if hd: quality = "[HD]"
                if season == -1: season = IPhoenix.getSeasonNumber(title)
                mid = ids
                try:
                    ep = int(stream[3].split(",")[-1])
                except:
                    ep = IPhoenix.getEpisodeNumber(title)
                if episode == None:
                    if not ep in eplist:
                        if ep > -1 and season > -1: mid = "S%02dE%02d" % (season,ep)
                        listEntry = {"path":"/seasondetails","title":title,"type":MetaData.typeEpisode,"season":season,"episode":ep,"sid":sid,"mid":mid,"args":{"url":url,"season":season,"episode":ep}}
                        if ids != None: 
                            listEntry["imdb_id"] = "tt"+ids
                        eplist.append(ep)
                        retList.append(listEntry)
                else:
                    if episode > -1 and season > -1: mid = "S%02dE%02d" % (season,episode)
                    if (episode > -1 and ep != -1 and episode==ep) or (episode==-1 or ep == -1):
                        title = "%s%s %s" % (language,quality,title)
                        
                        listEntry = {"path":"/hosterselection","title":title,"sid":sid,"mid":mid,"showFavourite":False,"args":{"hoster":hosters,"mid":mid,"sid":sid}}
                        # mid = media id , sid = series id
                        retList.append(listEntry)
                #print "id:%s,title:%s,language:%s,episode:%s,hd:%s" % (id,title,language,episode,hd)
        return retList;
    
    def getSeriesDetail(self,url):
        retList = []
        ids = self.getIds(url)
        imdb_id = ""
        if ids != None: 
            imdb_id = "tt"+ids
        sid = self.getSidByUrl(url)
        res = self.opener.open(url)
        title = self.getTitleFromSid(sid)
        if title != "": title = title+" - "
        html = res.read()
        s = BeautifulSoup(html)
        seasonselect = s("select")
        if seasonselect:
            for sel in seasonselect:
                if sel["id"].lower() == "season_list":
                    options = sel("option")
                    for option in options:
                        season = int(option["value"])
                        if season > 0:
                            _title = title+"S%02d"%season
                            retList.append({"path":"/seasondetails","title":_title,"sid":sid,"mid":"S%02d"%season,"season":str(season),"imdb_id":imdb_id,"type":MetaData.typeSeason,"args":{"url":url,"season":season}})
        return retList
    
    def getSeriesUpdates(self):
        retList = []
        res = self.opener.open("http://serienjunkies.org/xml/feeds/staffeln.xml")
        xml = res.read()
        s = BeautifulSoup(xml)
        items = s("item")
        if len(items) > 0:
            seriesList = []
            for item in items:
                url = item.contents[8]
                sid = self.getSidByUrl(url)
                if not sid in seriesList:
                    seriesList.append(sid)
                    title = self.getTitleFromSid(sid)
                    if title == "":
                        title = item.title.text.replace("DEUTSCH","DE").replace("ENGLISCH","EN").replace("ENLISCH","EN")
                        retList.append({"path":"/seriesdetail","title":title,"sid":sid,"mid":sid,"args":{"url":url}})
                    else:
                        retList.append({"path":"/seriesdetail","title":title,"sid":sid,"mid":sid,"type":MetaData.typeTVShow,"args":{"url":url}})
                    
        return retList;
    
    def getEpisodeUpdates(self):
        retList = []
        res = self.opener.open("http://serienjunkies.org/xml/feeds/episoden.xml")
        xml = res.read()
        s = BeautifulSoup(xml)
        items = s("item")
        if len(items) > 0:
            seriesList = []
            for item in items:
                url = item.contents[8]
                sid = self.getSidByUrl(url)
                if not sid in seriesList:
                    seriesList.append(sid)
                    title = self.getTitleFromSid(sid)
                    if title == "":
                        title = item.title.text.replace("DEUTSCH","DE").replace("ENGLISCH","EN").replace("ENLISCH","EN")
                        retList.append({"path":"/seriesdetail","title":title,"sid":sid,"mid":sid,"args":{"url":url}})
                    else:
                        retList.append({"path":"/seriesdetail","title":title,"sid":sid,"mid":sid,"type":MetaData.typeTVShow,"args":{"url":url}})
        return retList
    
    def getAZList(self,letter=None):
        cur = self.getDBCursor()
        retList = []
        if letter == None:
            for letter in self.AZ_DIRECTORIES:
                retList.append({"path":letter,"title":letter,"args":{"letter":letter}})
        elif letter != None:
            self.setHTTPCacheTimeout(86400) # cache a-z list 24hrs days
            res = self.opener.open("http://serienjunkies.org/?cat=0&l=%s" % letter.lower().replace("#","-"))
            self.resetHTTPCacheTimeout()
            html = res.read()
            s = BeautifulSoup(html)
            uls = s("ul")
            if len(uls) > 0:
                links = BeautifulSoup(str(uls[-1]))("a")
                for link in links:
                    url = link["href"]
                    sid = self.getSidByUrl(url)
                    ids = self.getIds(url,False)
                    imdb_id = ""
                    if ids != None:
                        imdb_id = "tt%s" % ids
                    title = link.text
                    retList.append({"path":"/seriesdetail","title":title,"sid":sid,"imdb_id":imdb_id,"type":MetaData.typeTVShow,"args":{"url":url}})
                    self.getDBCursor().execute("INSERT OR REPLACE INTO series (sid,title,url) VALUES(?,?,?)",(sid,title,url))
        return retList
    
    def search(self,searchstring):
        # Request URL:http://serienjunkies.org/media/ajax/search/search.php
        # Request Method:POST
        # string:gala
        retList = []
        res = self.opener.open("http://serienjunkies.org/media/ajax/search/search.php",urllib.urlencode({"string":searchstring}))
        jsonString = res.read()
        if jsonString != "":
            jsonData = json.loads(jsonString)
            for match in jsonData:
                retList.append({"path":"/seriesdetail","title":match[1],"sid":str(match[0]),"type":MetaData.typeTVShow,"showFavourite":False,"args":{"url":"http://serienjunkies.org/?cat=%s" % match[0]}})
        return retList
        
    def getSidByUrl(self,url):
        sid = url.split("/")[-1]
        if url.endswith("/"):
            sid = url.split("/")[-2]
        return sid
    
    def getTitleFromSid(self,sid):
        cur = self.getDBCursor()
        cur.execute("SELECT title FROM series WHERE sid=?",(sid,))
        titlerow = cur.fetchone()
        if titlerow != None:
            return titlerow[0]
        return ""
        
    def getIds(self,url,lookup=True):
        """ only scrapes for the id if needed otherwise uses integrted DB """
        cur = self.getDBCursor()
        cur.execute("SELECT ids FROM streamids WHERE url=? LIMIT 1", (url,))
        idsrow = cur.fetchone()
        if idsrow != None:
            return idsrow[0]
        elif lookup:
            res = self.opener.open(url)
            html = res.read()
            m = re.findall("{s:seasonNum,ids:'([0-9]+)'}",html,re.I + re.M + re.DOTALL) # TODO: cache ids for series links
            if len(m) > 0:
                ids = str(m[0])
                cur.execute('INSERT OR REPLACE INTO streamids (ids, url) VALUES (?,?)',(ids,url))
                return ids
        return None
    