import re
from urlparse import urlparse
from resolver import HosterException, StreamOfflineException, IPBlockedException, PluginException

class Resolver:
    opener = None
    cookieJar = None
    url = None
    hoster = None
    
    def __init__(self, url=None, domain=None, mid=None):
        '''
        Args:
            url (str): a URL to a web page that represents a piece of media.
            
            domain (str): the host of the media to be represented.
            
            mid (str): the unique ID given to the media by the host.
        '''
        if url == None and ( domain == None or mid == None ):
            raise HosterException('Set either url, or domain AND mid. No other combinations are valid.')
        
        try:
            if url != None:
                plugin = re.search("([^\.]+\.[^\.]+)$",urlparse(url).netloc).group(1)
            elif domain != None:
                plugin = re.search("([^\.]+\.[^\.]+)$",domain).group(1)
            
            plugin = plugin.replace(".","_").replace("-","")
            
            mod = __import__("resolver.hoster.%s" % plugin, fromlist=["resolver.hoster"])
            obj = getattr(mod, 'Hoster')
            self.hoster = obj(url=url,mid=mid)
            if self.hoster.getURL() != url or self.hoster.getMID() != mid:
                raise PluginException("Plugin does not invoke super constructor. Use Either super(Hoster, self).__init__() or IResolver.__init__() !!!!")
            
            if url != None and mid == None:
                self.hoster.parseURL(url)
                if self.hoster.getMID() == None:
                    raise PluginException("Plugin does not set mid. Use self.setMID(<mid>) in setupURL() of your plugin !!!!")
                elif self.hoster.getURL() == None:
                    raise PluginException("Plugin does not set url. Either use IResolver.setupURL(self, url) or self.setURL(<url>) in setupURL() of your plugin !!!!")
            else:
                self.hoster.setMID(mid)
            # timeout in seconds
        except IPBlockedException:
            raise
        except HosterException:
            raise
        except StreamOfflineException:
            raise
        
    def getHoster(self):
        return self.hoster

    def getStream(self):
        return self.hoster.resolve()
    


