from urlparse import urlparse
from cookielib import Cookie
from lib import IPhoenix

class IResolver(IPhoenix):    
    """ private BEGIN """
    __url = None
    __mid = None
    __title = None
    """ private END """
    
    """ public BEGIN """
    cachetimeout = 0 # disable http caching for resolvers 
    domain = None
    """ public END """
    
    """ Abstract Class for Resolving streams """
    def __init__(self,url=None,mid=None):
        IPhoenix.__init__(self)
        self.__url = url
        self.__mid = mid
    
    """### MUST BE IMPLEMENTED BEGIN BEGIN """
    
    def parseURL(self,url):
        """ sets up mid and url is optional based on given url """
        up = urlparse(url)
        if not self.domain in up.netloc:
            raise PluginException("plugin domain missmatch! [%s,%s]"%(self.domain,up.netloc))
        self.setURL(url)
    
    def resolve(self):
        """ return resolved stream URL """
        raise NotImplementedError("resolve(self,media_id)")
    
    """### MUST BE IMPLEMENTED BEGIN END """
    
    
    """### OPTIONAL IMPLEMENTATION BEGIN """
    
    def hasStaticStreamURL(self):
        """ return True if resolved stream urls stays always the same
            possible local caching of url in sqlite?
        """
        return False
    
    def hasLogin(self):
        """ return True if hoster supports premium logins """
        return False
    
    def doLogin(self,username=None,password=None):
        """ logs the User in with the given credentials """
        raise NotImplementedError("doLogin(self,username=None,password=None)")
    
    def isSSLSupported(self):
        return False
    
    """### OPTIONAL IMPLEMENTATION END """
    
    """### HELPER FUNCTIONALITY BEGIN """

    def getDomain(self):
        return self.domain
    
    def getURL(self):
        return self.__url
    
    def setURL(self,url):
        self.__url = url
        
    def getMID(self):
        return self.__mid
    
    def setMID(self,mid):
        self.__mid = mid
        
    def getTitle(self):
        return self.__title
        
    def setTitle(self,title):
        self.__title = title
    
    def getUsername(self,default=None):
        return default
    
    def getPassword(self,default=None):
        return default

    """### HELPER FUNCTIONALITY END """

class PluginException(Exception):
    pass

class HosterException(Exception):
    pass

class StreamOfflineException(Exception):
    pass

class IPBlockedException(Exception):
    pass